/*
 * x8042.c
 *
 * Expose the Xilinx PS/2 components as a typical PC 8042 keyboard controller
 *
 * Author: MontaVista Software, Inc.
 *         source@mvista.com
 *
 * 2002 (c) MontaVista, Software, Inc.  This file is licensed under the terms
 * of the GNU General Public License version 2.  This program is licensed
 * "as is" without any warranty of any kind, whether express or implied.
 */

/*
 * This code is a bit unusual in a few ways.  The first is that it is
 * emulating a hardware component: the typical PC's 8042 keyboard
 * controller.  With the current implementation of the keyboard code in
 * Linux, it seemed to make the most sense to not have yet another
 * hacked up pc_keyb.c to handle the unique interface that has been
 * exposed by Xilinx.  They basically expose the PS/2 interfaces just
 * like serial ports.
 *
 * This code defines four functions that are called by the typical PC
 * keyboard code when it wants to talk to the 8042.  These functions in
 * turn expose the Xilinx PS/2 interfaces via the typical 8042
 * interface.
 *
 * This brings us to another unusual aspect of this code.  Xilinx
 * provides drivers with their FPGA IP.  Their drivers are composed of
 * two logical parts where one part is the OS independent code and the
 * other part is the OS dependent code.  This file exposes their OS
 * independent functions as an 8042 keyboard controller.  The other
 * files in this directory are the OS independent files as provided by
 * Xilinx with no changes made to them.  The names exported by those
 * files begin with XPs2_.  All functions in this file that are called
 * by Linux have names that begin with x8042_.  The functions in this
 * file that have Handler in their name are registered as callbacks with
 * the underlying Xilinx OS independent layer.  Any other functions are
 * static helper functions.
 *
 * One more thing should be noted.  The words input and output can be
 * confusing in this context.  It depends upon whether you look at it
 * from the perspective of the main CPU or from the perspective of the
 * 8042 keyboard controller.  The KBD_STAT_OBF, KBD_STAT_MOUSE_OBF and
 * KBD_STAT_IBF defines from pc_keyb.h are from the perspective of the
 * 8042: OBF means Output Buffer Full which means that there is a byte
 * available to be read from the keyboard controller.  However,
 * pc_keyb.c also calls the functions kbd_read_input, kbd_read_status,
 * kbd_write_output and kbd_write_command.  The naming of these
 * functions is from the other perspective, that of the main CPU.
 *
 * There are some lurking issues that I have not addressed yet.  I haven't seen
 * them raise their heads, but I believe that the potential for a problem is
 * there.  The first issue is that nothing prevents the mouse from trashing the
 * keyboard and vice-versa.  I've tried to make this happen by holding down a
 * key while mucking with the mouse under X, but didn't see any problem, but it
 * still should probably be addressed.
 *
 * The second issue is that whenever we set an Output Buffer Full flag as the
 * result of a command, we should probably generate an interrupt.  I don't
 * think this is actually a problem because whenever pc_keyb.c sends a command
 * where a response is expected, it appears that it goes into a polling loop.
 * Something to remember though if you run into a problem.
 *
 * The final issue is that KBD_MODE_DISABLE_KBD and KBD_MODE_DISABLE_MOUSE are
 * not yet handled.  DISABLE_KBD is not used and DISABLE_MOUSE is only used
 * during startup and shutdown.  In short, this is another thing that should be
 * addressed.
 *
 * SAATODO: Address these issues.
 */

#include <linux/delay.h>
#include <linux/module.h>
#include <linux/ioport.h>
#include <linux/config.h>
#include <linux/init.h>
#include <linux/interrupt.h>
#include <linux/serio.h>
#include <linux/errno.h>
#include <linux/err.h>
#include <linux/platform_device.h>

#include <asm/io.h>
#include <asm/irq.h>

#include <xbasic_types.h>
#include <xparameters.h>
#include "xps2.h"
#include "xps2_i.h"

MODULE_AUTHOR("Vojtech Pavlik <vojtech@ucw.cz>");
MODULE_DESCRIPTION("Xilinx PS/2 IP core driver for Virtex FPGA family");
MODULE_LICENSE("GPL");

#define PS2_REMAP_SIZE		(XPAR_OPB_PS2_DUAL_REF_0_HIGHADDR_0 - \
				 XPAR_OPB_PS2_DUAL_REF_0_BASEADDR_0 + 1)

/* Our private per device data. */
struct xilinx_ps2_dev {
	int index;		/* Which interface is this */
	int irq;
	u32 phys_addr;		/* Saved physical base address */
	struct serio *serio;	/* Registered serio object for this instance */
	/*
	 * The underlying OS independent code needs space as well.  A
	 * pointer to the following XPs2 structure will be passed to
	 * any XPs2_ function that requires it.  However, we treat the
	 * data as an opaque object in this file (meaning that we never
	 * reference any of the fields inside of the structure).
	 */
	XPs2 Ps2;
};
static struct xilinx_ps2_dev keyboard = {
	.index = 0,
	.irq = XPAR_OPB_INTC_0_OPB_PS2_DUAL_REF_0_SYS_INTR1_INTR,
};
static struct xilinx_ps2_dev mouse = {
	.index = 1,
	.irq = XPAR_OPB_INTC_0_OPB_PS2_DUAL_REF_0_SYS_INTR2_INTR,
};

/* SAATODO: This function will be moved into the Xilinx code. */
/*****************************************************************************/
/**
*
* Lookup the device configuration based on the PS/2 instance.  The table
* XPs2_ConfigTable contains the configuration info for each device in the system.
*
* @param Instance is the index of the PS/2 interface being looked up.
*
* @return
*
* A pointer to the configuration table entry corresponding to the given
* device ID, or NULL if no match is found.
*
* @note
*
* None.
*
******************************************************************************/
static XPs2_Config*
XPs2_GetConfig(int Instance)
{
	if (Instance < 0 || Instance >= XPAR_XPS2_NUM_INSTANCES)
	{
		return NULL;
	}

	return &XPs2_ConfigTable[Instance];
}

static int
xilinx_ps2_open(struct serio *serio)
{
	struct xilinx_ps2_dev *dev = serio->port_data;
	printk(KERN_INFO "xilinx_ps2_open(serio=%p)\n", serio);
	XPs2_EnableInterrupt(&dev->Ps2);
	return 0;
}

static void
xilinx_ps2_close(struct serio *serio)
{
	//struct xilinx_ps2_dev *dev = serio->port_data;
	printk(KERN_INFO "xilinx_ps2_close(serio=%p)\n", serio);
	//XPs2_DisableInterrupt(&dev->Ps2);
}

static int
xilinx_ps2_write(struct serio *serio, unsigned char c)
{
	struct xilinx_ps2_dev *dev = serio->port_data;
	//printk(KERN_INFO "xilinx_ps2_write(serio=%p, c=%x)\n", serio, c);
	XPs2_Send(&dev->Ps2, &c, 1);
	return 0;
}

static irqreturn_t
xilinx_ps2_interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
	struct xilinx_ps2_dev *dev = dev_id;
	//printk(KERN_ERR "xilinx_ps2: irq(irq=%d, dev_id=%p)\n", irq, dev_id);
	XPs2_InterruptHandler(&dev->Ps2);
	return IRQ_HANDLED;
}

static void
xilinx_ps2_interrupt_handler(void *ref, u32 event, unsigned int data)
{
	struct xilinx_ps2_dev *dev = ref;
	u8 c = 0;

	switch (event) {
	case XPS2_EVENT_RECV_DATA:
		//printk(KERN_WARNING "xilinx_ps2: rx data %u\n", event);
		if (XPs2_Recv(&dev->Ps2, &c, 1) != 1) {
			printk(KERN_ERR "xilinx_ps2: missing rx data\n");
		}
		serio_interrupt(dev->serio, c, 0, NULL);
		break;

	case XPS2_EVENT_RECV_ERROR:
		serio_reconnect(dev->serio);
		break;

	case XPS2_EVENT_RECV_OVF:
		serio_reconnect(dev->serio);
		return;

	case XPS2_EVENT_SENT_DATA:
		return;

	default:
		printk(KERN_WARNING "xilinx_ps2: unknown event %u\n", event);
		return;
	}

}

static int
xilinx_ps2_probe(struct xilinx_ps2_dev *dev)
{
	struct platform_device *pdev;
	struct serio *serio;
	XPs2_Config *cfg;
	int rc;

	/* Find the config for our device. */
	cfg = XPs2_GetConfig(dev->index);
	if (!cfg) {
		printk(KERN_ERR
		       "xilinx_ps2: no config for device %d\n",
		       dev->index);
		return -ENODEV;
	}

	/* Map the device into virtual memory.  save physical address for
	 * restoring later. */
	dev->phys_addr = cfg->BaseAddress;
	cfg->BaseAddress = (u32) ioremap(dev->phys_addr, PS2_REMAP_SIZE);

	/* Initialize the device */
	if (XPs2_Initialize(&dev->Ps2, cfg->DeviceId) != XST_SUCCESS) {
		printk(KERN_ERR
		       "Xilinx PS/2 #%d: Could not initialize device.\n",
		       dev->index);
		return -ENODEV;
	}

	/* The device exists, create a platform device for it */
	pdev = platform_device_register_simple("xilinx_ps2", dev->index, NULL, 0);
	if (!pdev)
		return -ENOMEM;

	/* The serio stuct tells the input layer how to talk to the PS/2 device;
	 * Allocate and initialize it. */
	serio = kmalloc(sizeof(struct serio), GFP_KERNEL);
	if (!serio) {
		printk(KERN_ERR "xilinx_ps2: could not alloc serio struct\n");
		return -ENOMEM;
	}

	/* Initialize the serio structure */
	memset(serio, 0, sizeof(struct serio));
	serio->id.type = SERIO_8042;
	serio->open = xilinx_ps2_open;
	serio->close = xilinx_ps2_close;
	serio->write = xilinx_ps2_write;
	serio->port_data = dev;
	serio->dev.parent = &pdev->dev;
	snprintf(serio->name, sizeof(serio->name),
		 "Xilinx PS2 port %i", dev->index);
	snprintf(serio->phys, sizeof(serio->phys),
		 "xilinx/serio%i", dev->index);
	serio_register_port(serio);
	dev->serio = serio;

	/* Set up the interrupt handler. */
	rc = request_irq(dev->irq, xilinx_ps2_interrupt, 0, "xilinx_ps2", dev);
	if (rc) {
		printk(KERN_ERR "xilinx_ps2: Could not allocate interrupt %d.\n",
			dev->irq);
	}
	XPs2_SetHandler(&dev->Ps2, xilinx_ps2_interrupt_handler, dev);

	printk(KERN_INFO "Xilinx PS/2 #%d at 0x%08X mapped to 0x%08X\n",
	       dev->index, dev->phys_addr, cfg->BaseAddress);

	return 0;
}

static int __init
xilinx_ps2_init(void)
{
	int rc;
	printk(KERN_INFO "xilinx_ps2_init()\n");
	if ((rc = xilinx_ps2_probe(&mouse)))
		printk(KERN_ERR "xilinx_ps2: error initializing mouse (%d)\n", rc);
	if ((rc = xilinx_ps2_probe(&keyboard)))
		printk(KERN_ERR "xilinx_ps2: error initializing keyboard (%d)\n", rc);
	return 0;
}

static void __exit
xilinx_ps2_exit(void)
{
}

module_init(xilinx_ps2_init);
module_exit(xilinx_ps2_exit);
