/*
 * c67x00-udc.c: Cypress C67X00 USB device controller
 *
 * Copyright (C) 2006-2007 Barco N.V.
 *    Derived from the Cypress cy7c67200/300 ezusb linux driver and
 *    based on multiple device controller drivers inside the linux kernel.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA.
 */

#include <linux/device.h>
#include <linux/platform_device.h>
#include <linux/usb.h>
#include <linux/usb_gadget.h>

#include "c67x00.h"
#include "c67x00-udc.h"

#define MAX_NB_PERIPHERALS 2

struct c67x00_udc {
	spinlock_t lock;
	struct c67x00_sie *sie;
	struct usb_gadget_driver *driver;
};
#define udc_dev(udc) (sie_dev((udc)->sie))

/*
 * the sie and gadget_driver get probed/registered from 2 totally independant
 * places, this datastructure binds them together
 */
static struct c67x00_udc controller[MAX_NB_PERIPHERALS];

/* -------------------------------------------------------------------------- */

int usb_gadget_register_driver(struct usb_gadget_driver *driver)
{
	int i;
	struct c67x00_udc *udc = NULL;

	/* Find a free controller attached to an sie */
	for (i = 0; !udc && (i < MAX_NB_PERIPHERALS); i++) {
		udc = &controller[i];
		spin_lock(&udc->lock);
		if (!udc->sie || udc->driver) {
			spin_unlock(&udc->lock);
			udc = NULL;
		}
	}
	if (!udc)
		return -EBUSY;

	udc->driver = driver;
	/* TODO do something with driver->driver? */
	dev_dbg(udc_dev(udc), "Binding %s\n", driver->function);

	dev_warn(udc_dev(udc), "This is not implemented yet!\n");

	/* TODO bind */
	spin_unlock(&udc->lock);
	return 0;
}

EXPORT_SYMBOL(usb_gadget_register_driver);

int usb_gadget_unregister_driver(struct usb_gadget_driver *driver)
{
	int i;
	struct c67x00_udc *udc = NULL;

	for (i = 0; !udc && (i < MAX_NB_PERIPHERALS); i++) {
		udc = &controller[i];
		spin_lock(&udc->lock);
		if (udc->driver != driver) {
			spin_unlock(&udc->lock);
			udc = NULL;
		}
	}
	if (!udc)
		return -EINVAL;

	/* TODO unbind */
	udc->driver = NULL;

	spin_unlock(&udc->lock);
	return 0;
}

EXPORT_SYMBOL(usb_gadget_unregister_driver);

/* -------------------------------------------------------------------------- */

int c67x00_udc_probe(struct c67x00_sie *sie)
{
	int i;
	struct c67x00_udc *udc = NULL;

	/* Find an unused controller */
	for (i = 0; !udc && (i < MAX_NB_PERIPHERALS); i++) {
		udc = &controller[i];
		spin_lock(&udc->lock);
		if (udc->sie) {
			spin_unlock(&udc->lock);
			udc = NULL;
		}
	}
	if (!udc) {
		dev_warn(sie_dev(sie), "Could not find unused controller\n");
		return -EBUSY;
	}

	/* udc lock is held here */
	sie->private_data = udc;
	udc->sie = sie;

	spin_unlock(&udc->lock);
	return 0;
}

void c67x00_udc_remove(struct c67x00_sie *sie)
{
	struct c67x00_udc *udc = sie->private_data;
	if (!udc) {
		dev_err(sie_dev(sie), "No udc found!\n");
		return;
	}

	spin_lock(&udc->lock);
	/* TODO check if driver is register -> unbind? */
	sie->private_data = NULL;
	udc->sie = NULL;
	spin_unlock(&udc->lock);
}

/* -------------------------------------------------------------------------- */
/* Module initialization/cleanup */

int c67x00_udc_init(void)
{
	int i;
	struct c67x00_udc *udc;
	for (i = 0; i < MAX_NB_PERIPHERALS; i++) {
		udc = &controller[i];
		spin_lock_init(&udc->lock);
		udc->sie = NULL;	/* Mark unused */
	}
	return 0;
}

void c67x00_udc_cleanup(void)
{

}
