/*
 * c67x00.h: Cypress C67X00 USB register and field definitions
 *
 * Copyright (C) 2006-2007 Barco N.V.
 *    Derived from the Cypress cy7c67200/300 ezusb linux driver and
 *    based on multiple host controller drivers inside the linux kernel.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA.
 */

#ifndef _USB_C67X00_H
#define _USB_C67X00_H

#include <linux/interrupt.h>
#include <linux/list.h>
#include <linux/platform_device.h>
#include <linux/completion.h>
#include <linux/mutex.h>

/* ---------------------------------------------------------------------
 * Cypress C67x00 register definitions
 */

/* Processor control registers */
/* =========================== */

/* Hardware Revision Register */
#define HW_REV_REG		0xC004

/* General USB registers */
/* ===================== */

/* USB Control Register */
#define USB_CTL_REG(x)		( (x) ? 0xC0AA : 0xC08A )

#define LOW_SPEED_PORT(x)	( (x) ? 0x0800 : 0x0400 )
#define HOST_MODE		0x0200
#define PORT_RES_EN(x)		( (x) ? 0x0100 : 0x0080 )
#define SOF_EOP_EN(x)		( (x) ? 0x0002 : 0x0001 )

/* USB Host only registers */
/* ======================= */

/* Host n Control Register */
#define HOST_CTL_REG(x)		((x) ? 0xC0A0 : 0xC080)

#define PREAMBLE_EN		0x0080	/* Preamble enable */
#define SEQ_SEL			0x0040	/* Data Toggle Sequence Bit Select */
#define ISO_EN			0x0010	/* Isochronous enable  */
#define ARM_EN			0x0001	/* Arm operation */

/* Host n Interrupt Enable Register */
#define HOST_IRQ_EN_REG(x)	( (x) ? 0xC0AC : 0xC08C)

#define SOF_EOP_IRQ_EN		0x0200	/* SOF/EOP Interrupt Enable  */
#define DONE_IRQ_EN		0x0001	/* Done Interrupt Enable  */

/* Host n status register */
#define HOST_STAT_REG(x)	( (x) ? 0xC0B0 : 0xC090 )

#define HOST_STAT_MASK		0x02FD
#define SOF_EOP_IRQ_FLG		0x0200
#define PORT_CONNECT_CHANGE(x)	( (x) ? 0x0020 : 0x0010 )
#define PORT_SE0_STATUS(x)	( (x) ? 0x0008 : 0x0004 )

/* Host Frame Register */
#define HOST_FRAME_REG(x)	( (x) ? 0xC0B6 : 0xC096 )

#define HOST_FRAME_MASK		0x07FF

/* HPI registers */
/* ============= */

/* HPI Status register */
#define SOFEOP_FLG(x)		(1 << ( (x) ? 12 : 10 ))
#define SIEMSG_FLAG(x)		(1 << (4 + (x)))
#define MBX_OUT_FLG		0x0001	/* Message out available */

/* Interrupt routing register */
#define HPI_IRQ_ROUTING_REG	0x0142

#define SOFEOP_TO_HPI_EN(x)	( (x) ? 0x2000 : 0x0800 )
#define SOFEOP_TO_CPU_EN(x)	( (x) ? 0x1000 : 0x0400 )

/* SIE msg registers */
#define SIEMSG_REG(x)		( (x) ? 0x0148 : 0x0144 )

#define HUSB_TDListDone		0x1000

#define CY_HCD_BUF_ADDR		0x500	/* Base address for host */
#define SIE_TD_SIZE		0x200	/* size of the td list */
#define SIE_TD_BUF_SIZE		0x400	/* size of the data buffer */

#define SIE_TD_OFFSET(host)	( (host) ? (SIE_TD_SIZE+SIE_TD_BUF_SIZE) : 0 )
#define SIE_BUF_OFFSET(host)	(SIE_TD_OFFSET(host) + SIE_TD_SIZE)

/* ---------------------------------------------------------------------
 * Driver data structures
 */

struct c67x00_device;

/**
 * struct c67x00_lcp
 */
struct c67x00_lcp {
	/* Internal use only */
	struct mutex mutex;
	struct completion msg_received;
	u16 last_msg;
};

/**
 * struct c67x00_lcp_data
 */
#define COMM_REGS 14
struct c67x00_lcp_int_data {
	u16 regs[COMM_REGS];
};

/**
 * struct c67x00_sie - Common data associated with an SIE
 * @lock: lock to protect this struct
 * @private_data: subdriver dependent data
 * @pdev: platform device associated with this SIE, created in c67x00-drv.c
 * @irq: subdriver depenent irq handler, set NULL when not used
 * @msg_received: called when an SIEmsg has been received
 * @dev: link to common driver structure
 * @sie_num: SIE number on chip, starting from 0
 * @mode: SIE mode (host/peripheral/otg/not used)
 *
 * Each SIE has a separate platform_device associated with it, because the
 * hcd needs such a device for itself (TODO: use a struct device instead
 * of a new platform device).
 */
struct c67x00_sie {
	/* Entries to be used by the subdrivers */
	spinlock_t lock;	/* protect this structure */
	void *private_data;
	struct platform_device *pdev;
	irqreturn_t(*irq) (int irq, struct c67x00_sie * sie);
	void (*msg_received) (struct c67x00_sie * sie, u16 msg);

	/* Read only: */
	struct c67x00_device *dev;
	int sie_num;
	int mode;
};

#define sie_dev(s)	(&(s)->pdev->dev)

struct c67x00_hpi {
	void __iomem *base;
	int regstep;
};

#define C67X00_SIES	2
#define C67X00_PORTS	2

/**
 * struct c67x00_device - Common data structure for a c67x00 instance
 * @hpi: hpi addresses
 * @sie: array of sie's on this chip
 * @pdata: configuration provided by the platform
 * @hw_lock: hardware lock
 * @int_status: interrupt status register, only valid in_interrupt()
 * @lcp: link control protocol dependent data
 */
struct c67x00_device {
	struct c67x00_hpi hpi;
	struct c67x00_sie sie[C67X00_SIES];
	struct platform_device *pdev;
	struct c67x00_platform_data *pdata;
	spinlock_t hw_lock;

	u16 int_status;
	struct c67x00_lcp lcp;
};

/* ---------------------------------------------------------------------
 * Low level interface functions
 */

/* Host Port Interface (hpi) functions */
u16 c67x00_ll_hpi_status(struct c67x00_device *dev);
void c67x00_ll_hpi_reg_init(struct c67x00_device *dev);
void c67x00_ll_hpi_enable_sofeop(struct c67x00_sie *sie);
void c67x00_ll_hpi_disable_sofeop(struct c67x00_sie *sie);

/* General functions */
u16 c67x00_ll_get_siemsg(struct c67x00_device *dev, int sie);
void c67x00_ll_set_siemsg(struct c67x00_device *dev, int sie, u16 val);
u16 c67x00_ll_get_usb_ctl(struct c67x00_sie *sie);

/* Host specific functions */
void c67x00_ll_set_husb_eot(struct c67x00_device *dev, u16 value);
void c67x00_ll_husb_reset(struct c67x00_sie *sie, int port);
void c67x00_ll_husb_set_current_td(struct c67x00_sie *sie, u16 addr);
u16 c67x00_ll_husb_get_current_td(struct c67x00_sie *sie);
void c67x00_ll_husb_clear_status(struct c67x00_sie *sie, u16 bits);
u16 c67x00_ll_husb_get_status(struct c67x00_sie *sie);
u16 c67x00_ll_husb_get_frame(struct c67x00_sie *sie);
void c67x00_ll_husb_init_host_port(struct c67x00_sie *sie);
void c67x00_ll_husb_reset_port(struct c67x00_sie *sie, int port);

/* Slave specific functions */
void c67x00_ll_susb_init(struct c67x00_sie *sie);

/* Read and write to memory */
void c67x00_ll_hpi_write_mem_le16(struct c67x00_device *dev, u16 addr,
				  int len, char *data);
void c67x00_ll_hpi_read_mem_le16(struct c67x00_device *dev, u16 addr,
				 int len, char *data);

/* Called by c67x00_irq to handle lcp interrupts */
void c67x00_ll_irq(struct c67x00_device *dev);

/* Setup and teardown */
void c67x00_ll_init(struct c67x00_device *dev);
void c67x00_ll_release(struct c67x00_device *dev);
int c67x00_ll_reset(struct c67x00_device *dev);

#endif				/* _USB_C67X00_H */
